@@ -33,9 +33,9 @@
 #include "ktoolbar.h"
 
 #include <QApplication>
-#include <QtCore/QList>
-#include <QtCore/QObject>
-#include <QtCore/QTimer>
+#include <QList>
+#include <QObject>
+#include <QTimer>
 #include <QCloseEvent>
 #include <QDesktopWidget>
 #include <QDockWidget>
@@ -84,8 +84,8 @@ class DockResizeListener : public QObject
 {
 public:
     DockResizeListener(KMainWindow *win);
-    virtual ~DockResizeListener();
-    bool eventFilter(QObject *watched, QEvent *event) Q_DECL_OVERRIDE;
+    ~DockResizeListener() override;
+    bool eventFilter(QObject *watched, QEvent *event) override;
 
 private:
     KMainWindow *m_win;
