@@ -24,8 +24,10 @@
 */
 
 #include "kxmlguiwindow.h"
+#include "debug.h"
 
 #include "kmainwindow_p.h"
+#include "kmessagebox.h"
 #include "kactioncollection.h"
 #include "kmainwindowiface_p.h"
 #include "ktoolbarhandler_p.h"
@@ -36,9 +38,8 @@
 
 #include <QCloseEvent>
 #include <QDBusConnection>
-#include <QtXml/QDomDocument>
+#include <QDomDocument>
 #include <QLayout>
-#include <QDebug>
 #include <QMenuBar>
 #include <QObject>
 #include <QStatusBar>
@@ -83,9 +84,9 @@ KXmlGuiWindow::KXmlGuiWindow(QWidget *parent, Qt::WindowFlags f)
 {
     K_D(KXmlGuiWindow);
     d->showHelpMenu = true;
-    d->toolBarHandler = 0;
-    d->showStatusBarAction = 0;
-    d->factory = 0;
+    d->toolBarHandler = nullptr;
+    d->showStatusBarAction = nullptr;
+    d->factory = nullptr;
     new KMainWindowInterface(this);
 }
 
@@ -93,7 +94,7 @@ QAction *KXmlGuiWindow::toolBarMenuAction()
 {
     K_D(KXmlGuiWindow);
     if (!d->toolBarHandler) {
-        return 0;
+        return nullptr;
     }
 
     return d->toolBarHandler->toolBarMenuAction();
@@ -249,6 +250,7 @@ void KXmlGuiWindow::createGUI(const QString &xmlfile)
         QAction *switchLanguageAction = d->helpMenu->action(KHelpMenu::menuSwitchLanguage);
         QAction *aboutAppAction = d->helpMenu->action(KHelpMenu::menuAboutApp);
         QAction *aboutKdeAction = d->helpMenu->action(KHelpMenu::menuAboutKDE);
+        QAction *donateAction = d->helpMenu->action(KHelpMenu::menuDonate);
 
         if (helpContentsAction) {
             actions->addAction(helpContentsAction->objectName(), helpContentsAction);
@@ -268,13 +270,16 @@ void KXmlGuiWindow::createGUI(const QString &xmlfile)
         if (aboutKdeAction) {
             actions->addAction(aboutKdeAction->objectName(), aboutKdeAction);
         }
+        if (donateAction) {
+            actions->addAction(donateAction->objectName(), donateAction);
+        }
     }
 
     const QString windowXmlFile = xmlfile.isNull() ? componentName() + QStringLiteral("ui.rc") : xmlfile;
 
     // Help beginners who call setXMLFile and then setupGUI...
     if (!xmlFile().isEmpty() && xmlFile() != windowXmlFile) {
-        qWarning() << "You called setXMLFile(" << xmlFile() << ") and then createGUI or setupGUI,"
+        qCWarning(DEBUG_KXMLGUI) << "You called setXMLFile(" << xmlFile() << ") and then createGUI or setupGUI,"
                    << "which also calls setXMLFile and will overwrite the file you have previously set.\n"
                    << "You should call createGUI(" << xmlFile() << ") or setupGUI(<options>," << xmlFile() << ") instead.";
     }
@@ -292,6 +297,8 @@ void KXmlGuiWindow::createGUI(const QString &xmlfile)
     guiFactory()->reset();
     guiFactory()->addClient(this);
 
+    checkAmbiguousShortcuts();
+
     //  setUpdatesEnabled( true );
 }
 
@@ -330,7 +337,7 @@ void KXmlGuiWindow::setStandardToolBarMenuEnabled(bool enable)
         }
 
         delete d->toolBarHandler;
-        d->toolBarHandler = 0;
+        d->toolBarHandler = nullptr;
     }
 }
 
@@ -350,7 +357,7 @@ void KXmlGuiWindow::createStandardStatusBarAction()
         d->showStatusBarAction->setChecked(sb->isHidden());
     } else {
         // If the language has changed, we'll need to grab the new text and whatsThis
-        QAction *tmpStatusBar = KStandardAction::showStatusbar(NULL, NULL, NULL);
+        QAction *tmpStatusBar = KStandardAction::showStatusbar(nullptr, nullptr, nullptr);
         d->showStatusBarAction->setText(tmpStatusBar->text());
         d->showStatusBarAction->setWhatsThis(tmpStatusBar->whatsThis());
         delete tmpStatusBar;
@@ -385,5 +392,50 @@ void KXmlGuiWindow::finalizeGUI(KXMLGUIClient *client)
     KXMLGUIBuilder::finalizeGUI(client);
 }
 
+void KXmlGuiWindow::checkAmbiguousShortcuts()
+{
+    QMap<QString, QAction*> shortcuts;
+    QAction *editCutAction = actionCollection()->action(QStringLiteral("edit_cut"));
+    QAction *deleteFileAction = actionCollection()->action(QStringLiteral("deletefile"));
+    foreach (QAction *action, actionCollection()->actions()) {
+        if (action->isEnabled()) {
+            foreach (const QKeySequence &shortcut, action->shortcuts()) {
+                if (shortcut.isEmpty()) {
+                    continue;
+                }
+                const QString portableShortcutText = shortcut.toString();
+                const QAction *existingShortcutAction = shortcuts.value(portableShortcutText);
+                if (existingShortcutAction) {
+                    // If the shortcut is already in use we give a warning, so that hopefully the developer will find it
+                    // There is one exception, if the conflicting shortcut is a non primary shortcut of "edit_cut"
+                    // and "deleteFileAction" is the other action since Shift+Delete is used for both in our default code
+                    bool showWarning = true;
+                    if ((action == editCutAction && existingShortcutAction == deleteFileAction) ||
+                        (action == deleteFileAction && existingShortcutAction == editCutAction)) {
+                        QList<QKeySequence> editCutActionShortcuts = editCutAction->shortcuts();
+                        if (editCutActionShortcuts.indexOf(shortcut) > 0) // alternate shortcut
+                        {
+                            editCutActionShortcuts.removeAll(shortcut);
+                            editCutAction->setShortcuts(editCutActionShortcuts);
+
+                            showWarning = false;
+                        }
+                    }
+
+                    if (showWarning) {
+                        const QString actionName = KLocalizedString::removeAcceleratorMarker(action->text());
+                        const QString existingShortcutActionName = KLocalizedString::removeAcceleratorMarker(existingShortcutAction->text());
+                        QString dontShowAgainString = existingShortcutActionName + actionName + shortcut.toString();
+                        dontShowAgainString.remove(QLatin1Char('\\'));
+                        KMessageBox::information(this, i18n("There are two actions (%1, %2) that want to use the same shortcut (%3). This is most probably a bug. Please report it in <a href='https://bugs.kde.org'>bugs.kde.org</a>", existingShortcutActionName, actionName, shortcut.toString(QKeySequence::NativeText)), i18n("Ambiguous Shortcuts"), dontShowAgainString, KMessageBox::Notify | KMessageBox::AllowLink);
+                    }
+                } else {
+                    shortcuts.insert(portableShortcutText, action);
+                }
+            }
+        }
+    }
+}
+
 #include "moc_kxmlguiwindow.cpp"
 
